from .request_body import (
    Options,
    RecommendationOptions,
    RequestAutomaticCompressionParams,
    RequestAvailableLayers,
    RequestCreateCompression,
    RequestCreateRecommendation,
    RequestUpdateCompression,
)
from .response_body import (
    ResponseCompression,
    ResponseCompressionItem,
    ResponseCompressionItems,
    ResponseRecommendationItem,
    ResponseSelectMethod,
    ResponseSelectMethodItem,
)

__all__ = [
    Options,
    RecommendationOptions,
    RequestCreateCompression,
    RequestCreateRecommendation,
    RequestUpdateCompression,
    RequestAutomaticCompressionParams,
    RequestAvailableLayers,
    ResponseCompressionItem,
    ResponseCompressionItems,
    ResponseRecommendationItem,
    ResponseSelectMethodItem,
    ResponseSelectMethod,
    ResponseCompression,
]
